// Extension for exporting maps from Tiled Map Editor by Johan Kårlin v 1.2
// The map is saved in a text format that ACME Cross-Assembler can read.

var acmeMapFormat = {
    name: "X16 ACME Asm",
    extension: "asm",

    write: function(map, fileName) {
		// Ask user for palette offset
		var paletteOffset = parseInt(tiled.prompt("Palette Offset:", "0"));
	
		// Create file and write width and height
        var file = new TextFile(fileName, TextFile.WriteOnly);
		file.writeLine("mapWidth:\t!byte " + map.width);
		file.writeLine("mapHeight:\t!byte " + map.height + "\n");

		// Write each tile layer in map
        for (var i = 0; i < map.layerCount; ++i) {
            var layer = map.layerAt(i);
			file.writeLine(layer.name.replace(/ /g,"_") + "_" + i + ":");
            if (layer.isTileLayer) {
				
                // Write each row in current layer
				for (y = 0; y < layer.height; ++y) {
                    var row = "\t!word ";
                    
					// Write each column in current layer
					for (x = 0; x < layer.width; ++x) {
						var cell = layer.cellAt(x, y);
						// Build a 16 bit hex value for each tile
						var val = (paletteOffset << 12) + (cell.tileId & 1023);
						if (cell.flippedVertically) {
							val += 0x0800; 
						}
						if (cell.flippedHorizontally) {
							val += 0x0400;
						}
						// let each value be in the format "$xxxx"
                        row += "$" + ("000" + val.toString(16)).slice(-4) + ",";
                    }
					row = row.slice(0,-1);
					file.writeLine(row);
                }
				file.writeLine("");
            }
        }
        file.commit();
    },
}

tiled.registerMapFormat("X16 ACME Asm", acmeMapFormat)