// Extension for exporting maps int Tiled Map Editor by Johan Kårlin v 1.1
// The map is saved in a raw binary format.

var binaryMapFormat = {
    name: "X16 Binary",
    extension: "bin",

    write: function(map, fileName) {
		// Ask user for palette offset
		var paletteOffset = parseInt(tiled.prompt("Palette Offset:", "0"));
	
		// Create file and write width and height
        var file = new BinaryFile(fileName, BinaryFile.WriteOnly);
		var buffer = new ArrayBuffer(2);
		var words = new Uint16Array(buffer);
		file.write(buffer); // header will contain no information

		// Write each tile layer in map
        for (var i = 0; i < map.layerCount; ++i) {
            var layer = map.layerAt(i);
            if (layer.isTileLayer) {
				
                // Write each row in current layer
				for (y = 0; y < layer.height; ++y) {
                    
					// Write each column in current layer
					for (x = 0; x < layer.width; ++x) {
						var cell = layer.cellAt(x, y);
						// Build and write 16 bit word with information about palette, flipping and 10 bit tile id
						var word = (paletteOffset << 12) + (cell.tileId & 1023);
						if (cell.flippedVertically) {
							word += 0x0800; 
						}
						if (cell.flippedHorizontally) {
							word += 0x0400;
						}
						words[0] = word;
						file.write(buffer);
                    }
                }
            }
        }
        file.commit();
    },
}

tiled.registerMapFormat("X16 Binary", binaryMapFormat)